#include <HardwareProfile.h>
#include <plib.h>
#include <calculations.h>

// Definitions //////////////////////////////////////////////
#define DESIRED_BAUDRATE       19200      // The desired BaudRate RS-232
#define Fsck   					50000 
#define BRG_VAL    				((SYS_FREQ/2/Fsck)-2) 

// Global variables /////////////////////////////////////////
int phi1 = 1300; // tenths of a degree
int phi2 = 130; // tenths of a degree
int oldup1 = 0; // store old timer values to check for rollover
int oldup2 = 0;
int olddn1 = 0;
int olddn2 = 0;


// Function declarations ////////////////////////////////////
void initEncoder();
void getEncoderPosition1();
void getEncoderPosition2();
void initUART1();
void initInterruptComm();
void MAX518(int  force);


// Main function ////////////////////////////////////////////
int main(void)
{
     // Allow vector interrupts
     INTEnableSystemMultiVectoredInt();
   
   	// Initialize things
    	initEncoder();
    	mInitAllLEDs();
    	initUART1();
    	initInterruptComm();

    	while(1)
    	{ 
	    	getEncoderPosition1();
         	getEncoderPosition2();
      	forwardKinematics(phi1, phi2);
   	}
}


// Functions /////////////////////////////////////////////////
void initEncoder(void)
{
   // init Timer2 to Timer5 mode and periods (PR4, PR5)
   OpenTimer2( T2_ON | T2_PS_1_1 | T2_SOURCE_EXT, 0xFFFF); 
   OpenTimer3( T3_ON | T3_PS_1_1 | T3_SOURCE_EXT, 0xFFFF); 
   OpenTimer4( T4_ON | T4_PS_1_1 | T4_SOURCE_EXT, 0xFFFF); 
   OpenTimer5( T5_ON | T5_PS_1_1 | T5_SOURCE_EXT, 0xFFFF); 
} 


void getEncoderPosition1()
{
   int up1 = ReadTimer2();
   int dn1 = ReadTimer3();
   int cn1 = (up1 - dn1) - (oldup1 - olddn1);       //cn1 donates the difference

   if (up1 < oldup1){
      cn1 += 65536; // count0 only increments, so if it got lower it must have rolled over
   }
   if (dn1 < olddn1){
      cn1 -= 65536;
   }

   oldup1 = up1;
   olddn1 = dn1;

   phi1 += (cn1); // update angle
}

void   getEncoderPosition2()
{
   int up2 = ReadTimer4();
   int dn2 = ReadTimer5();
   int cn2 = (up2 - dn2) - (oldup2 - olddn2);       //cn1 donates the difference

   if (up2 < oldup2){
      cn2 += 65536; // count0 only increments, so if it got lower it must have rolled over
   }
   if (dn2 < olddn2){
      cn2 -= 65536;
   }
   oldup2 = up2;
   olddn2 = dn2;
   
   phi2 += (cn2); // update angle
}

void initUART1(int pbClk)
{
    // define setup Configuration 1 for OpenUARTx
       // Module Enable 
       // Work in IDLE mode 
       // Communication through usual pins 
       // Disable wake-up 
       // Loop back disabled 
       // Input to Capture module from ICx pin 
       // no parity 8 bit 
       // 1 stop bit 
       // IRDA encoder and decoder disabled 
       // CTS and RTS pins are disabled 
       // UxRX idle state is '1' 
       // 16x baud clock - normal speed
    #define config1    UART_EN | UART_IDLE_CON | UART_RX_TX | UART_DIS_WAKE | UART_DIS_LOOPBACK | UART_DIS_ABAUD | UART_NO_PAR_8BIT | UART_1STOPBIT | UART_IRDA_DIS | UART_DIS_BCLK_CTS_RTS| UART_NORMAL_RX | UART_BRGH_SIXTEEN
    
    // define setup Configuration 2 for OpenUARTx
       // IrDA encoded UxTX idle state is '0'
       // Enable UxRX pin
       // Enable UxTX pin
       // Interrupt on transfer of every character to TSR 
       // Interrupt on every char received
       // Disable 9-bit address detect
       // Rx Buffer Over run status bit clear
    #define config2      UART_TX_PIN_LOW | UART_RX_ENABLE | UART_TX_ENABLE | UART_INT_TX | UART_INT_RX_CHAR | UART_ADR_DETECT_DIS | UART_RX_OVERRUN_CLEAR   
 
    // Open UART2 with config1 and config2
    OpenUART1( config1, config2, SYS_FREQ/16/DESIRED_BAUDRATE-1);   // calculate actual BAUD generate value.
       
    // Configure UART2 RX Interrupt with priority 2
    ConfigIntUART1(UART_INT_PR2 | UART_RX_INT_EN);
}


void initInterruptComm()
{
	// Initialize timer 1 to generate interrupt to send data to PC
    // init Timer1 mode and period (PR1) // produces 10ms period
    OpenTimer1( T1_ON | T1_PS_1_64 | T1_SOURCE_INT, 12499); 
      
    mT1SetIntPriority(7);    // set Timer3 Interrupt Priority
    mT1ClearIntFlag();       // clear interrupt flag
    mT1IntEnable(1);      // enable timer3 interrupts

	// Initialize SPI lines to communicate data to DACs
	OpenI2C1( I2C_EN, BRG_VAL );
	OpenI2C2( I2C_EN, BRG_VAL );
}


void MAX518(int force)
{
	unsigned char addr = 0b01011000; // address of DAC on I2C line
	unsigned char cmd = 0x00; //command line 
	int force1 = 127+(7-(force >> 4))*8; // upper nibble to motor 1
	int force2 = 127+(7-(force & 15))*8; // lower nibble to motor 2
    
	// Send to DAC 1 //////////////////////
	StartI2C1();   //Send the Start Bit (begin of data send) 
    IdleI2C1();      //Wait to complete 

    MasterWriteI2C1 (addr); //address 
    IdleI2C1(); 
    while( !I2C1STATbits.ACKSTAT==0 ) {}

    MasterWriteI2C1 (cmd); //command line 
    IdleI2C1(); 
    while( !I2C1STATbits.ACKSTAT==0 ) {}

    MasterWriteI2C1(force1); //output 
    IdleI2C1(); 
    while( !I2C1STATbits.ACKSTAT==0 ) {}

    StopI2C1();  //end of data send 
    IdleI2C1();      //Wait to complete 


	// Send to DAC 2 //////////////////////
    StartI2C2();   //Send the Start Bit (begin of data send) 
    IdleI2C2();      //Wait to complete 

    MasterWriteI2C2 (addr); //address 
    IdleI2C2(); 
    while( !I2C2STATbits.ACKSTAT==0 ) {}
      
    MasterWriteI2C2 (cmd); //command line 
    IdleI2C2(); 
    while( !I2C2STATbits.ACKSTAT==0 ) {}

    MasterWriteI2C2(force2); //output 
    IdleI2C2(); 
    while( !I2C2STATbits.ACKSTAT==0 ) {}

    StopI2C2();  //end of data send 
    IdleI2C2();      //Wait to complete 
} 


//Interrupt Handlers////////////////////////////////////////////
// UART 1 interrupt handler
// it is set at priority level 2
void __ISR(_UART1_VECTOR, ipl2) IntUart1Handler(void)
{
   unsigned char force;

   // Is this an RX interrupt?
   if(mU1RXGetIntFlag()) {
        // Clear the RX interrupt Flag
        mU1RXClearIntFlag();
   
		force = ReadUART1(); // get data from PC
	    mLED_0_Toggle(); // toggle LED for feedback
		MAX518(force); // send signal to motors
	}

   // We don't care about TX interrupt
   if ( mU1TXGetIntFlag() ) {
        mU1TXClearIntFlag();
   }
}


// interrput code for the timer 1
void __ISR( _TIMER_1_VECTOR, ipl7) T1Interrupt( void)
{
	// send data to PC every 10 ms (based on Timer1
    char RS232_Out_Buffer[32];
    sprintf(RS232_Out_Buffer,"%d %d %d %d\n",X, Y, phi1, phi2);
	putsUART1(RS232_Out_Buffer);

	mLED_3_Toggle(); // toggle LED for feedback
   
    // clear interrupt flag and exit
    mT1ClearIntFlag();
} // T1 Interrupt


